"""DequeDict - Ordered dictionary with O(1) deque operations at both ends."""
from __future__ import annotations

import os
from collections.abc import Iterable, Mapping
from contextlib import suppress
from typing import TYPE_CHECKING, Callable, Generic, ItemsView, Iterator, KeysView, TypeVar, ValuesView, overload

from typing_extensions import TypeIs

__all__ = ["DequeDict", "DefaultDequeDict"]

K = TypeVar("K")
V = TypeVar("V")


def _is_iterable_of_pairs(items: object) -> TypeIs[Iterable[tuple[K, V]]]:
    return not isinstance(items, Mapping) and getattr(items, "__iter__", None) is not None


class DequeDict(Mapping[K, V]):
    """Ordered dictionary with O(1) deque operations at both ends.

    Combines dict interface with deque-like operations:
    - O(1) key lookup, contains, delete
    - O(1) popleft/popleftitem/peekleft/peekleftitem
    - O(1) pop/popitem/peek/peekitem
    - O(1) appendleft
    - O(1) move_to_end
    """

    __slots__ = ("_dict", "_head", "_tail")
    __hash__ = None  # type: ignore[assignment]

    class _Node:
        __slots__ = ("key", "value", "prev", "next")

        def __init__(self, key: K, value: V) -> None:
            self.key = key
            self.value = value
            self.prev: DequeDict._Node | None = None
            self.next: DequeDict._Node | None = None

    def __init__(self, items: Mapping[K, V] | Iterable[tuple[K, V]] | None = None) -> None:
        self._dict: dict[K, DequeDict._Node] = {}
        self._head: DequeDict._Node | None = None
        self._tail: DequeDict._Node | None = None
        if items is not None:
            if _is_iterable_of_pairs(items):
                for k, v in items:
                    self[k] = v
            else:
                for k, v in items.items():
                    self[k] = v

    def __len__(self) -> int:
        return len(self._dict)

    def __contains__(self, key: object) -> bool:
        return key in self._dict

    def __getitem__(self, key: K) -> V:
        if key not in self._dict:
            raise KeyError(key)
        return self._dict[key].value

    def __setitem__(self, key: K, value: V) -> None:
        if key in self._dict:
            self._dict[key].value = value
            return
        node = self._Node(key, value)
        self._dict[key] = node
        if self._tail is None:
            self._head = self._tail = node
        else:
            node.prev = self._tail
            self._tail.next = node
            self._tail = node

    def __delitem__(self, key: K) -> None:
        if key not in self._dict:
            raise KeyError(key)
        node = self._dict.pop(key)
        self._unlink(node)

    def _unlink(self, node: _Node) -> None:
        if node.prev:
            node.prev.next = node.next
        else:
            self._head = node.next
        if node.next:
            node.next.prev = node.prev
        else:
            self._tail = node.prev

    def __iter__(self) -> Iterator[K]:
        node = self._head
        while node:
            yield node.key
            node = node.next

    def __reversed__(self) -> Iterator[K]:
        node = self._tail
        while node:
            yield node.key
            node = node.prev

    def __repr__(self) -> str:
        if not self._dict:
            return "DequeDict()"
        items = [(n.key, n.value) for n in self._iter_nodes()]
        return f"DequeDict({items!r})"

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, (dict, DequeDict)):
            return NotImplemented
        if len(self) != len(other):
            return False
        return all(not (k not in other or other[k] != v) for k, v in self.items())

    def _iter_nodes(self) -> Iterator[_Node]:
        node = self._head
        while node:
            yield node
            node = node.next

    def peekleft(self) -> V:
        """Return first value without removing."""
        if self._head is None:
            raise IndexError("peek from an empty DequeDict")
        return self._head.value

    def peekleftitem(self) -> tuple[K, V]:
        """Return first (key, value) without removing."""
        if self._head is None:
            raise IndexError("peek from an empty DequeDict")
        return (self._head.key, self._head.value)

    def peekleftkey(self) -> K:
        """Return first key without removing."""
        if self._head is None:
            raise IndexError("peek from an empty DequeDict")
        return self._head.key

    def peek(self) -> V:
        """Return last value without removing."""
        if self._tail is None:
            raise IndexError("peek from an empty DequeDict")
        return self._tail.value

    def peekitem(self) -> tuple[K, V]:
        """Return last (key, value) without removing."""
        if self._tail is None:
            raise IndexError("peek from an empty DequeDict")
        return (self._tail.key, self._tail.value)

    def popleft(self) -> V:
        """Remove and return first value."""
        if self._head is None:
            raise IndexError("pop from an empty DequeDict")
        node = self._head
        del self._dict[node.key]
        self._unlink(node)
        return node.value

    def popleftitem(self) -> tuple[K, V]:
        """Remove and return first (key, value)."""
        if self._head is None:
            raise KeyError("popleftitem from an empty DequeDict")
        node = self._head
        del self._dict[node.key]
        self._unlink(node)
        return (node.key, node.value)

    @overload
    def pop(self) -> V: ...
    @overload
    def pop(self, key: K) -> V: ...
    @overload
    def pop(self, key: K, default: V) -> V: ...

    def pop(self, key: K | None = None, default: V | None = None) -> V:  # type: ignore[misc]
        """Remove and return value by key, or from end if no key given."""
        if key is None:
            if self._tail is None:
                if default is not None:
                    return default
                raise IndexError("pop from an empty DequeDict")
            node = self._tail
            del self._dict[node.key]
            self._unlink(node)
            return node.value
        if key not in self._dict:
            if default is not None:
                return default
            raise KeyError(key)
        node = self._dict.pop(key)
        self._unlink(node)
        return node.value

    def popitem(self) -> tuple[K, V]:
        """Remove and return last (key, value)."""
        if self._tail is None:
            raise KeyError("popitem from an empty DequeDict")
        node = self._tail
        del self._dict[node.key]
        self._unlink(node)
        return (node.key, node.value)

    def appendleft(self, key: K, value: V) -> None:
        """Insert (key, value) at front."""
        if key in self._dict:
            raise KeyError("key already exists")
        node = self._Node(key, value)
        self._dict[key] = node
        if self._head is None:
            self._head = self._tail = node
        else:
            node.next = self._head
            self._head.prev = node
            self._head = node

    def move_to_end(self, key: K, last: bool = True) -> None:
        """Move existing key to front (last=False) or back (last=True)."""
        if key not in self._dict:
            raise KeyError(key)
        node = self._dict[key]
        if (last and node is self._tail) or (not last and node is self._head):
            return
        self._unlink(node)
        if last:
            node.prev = self._tail
            node.next = None
            if self._tail:
                self._tail.next = node
            else:
                self._head = node
            self._tail = node
        else:
            node.prev = None
            node.next = self._head
            if self._head:
                self._head.prev = node
            else:
                self._tail = node
            self._head = node

    def get(self, key: K, default: V | None = None) -> V | None:
        """Return value for key, or default if key not present."""
        if key not in self._dict:
            return default
        return self._dict[key].value

    def keys(self) -> KeysView[K]:
        """Return view of keys in insertion order."""
        return _DequeDictKeysView(self)

    def values(self) -> ValuesView[V]:
        """Return view of values in insertion order."""
        return _DequeDictValuesView(self)

    def items(self) -> ItemsView[K, V]:
        """Return view of (key, value) pairs in insertion order."""
        return _DequeDictItemsView(self)

    def clear(self) -> None:
        """Remove all items."""
        self._dict.clear()
        self._head = None
        self._tail = None

    def copy(self) -> DequeDict[K, V]:
        """Return a shallow copy."""
        return DequeDict(self.items())

    def update(self, other: Mapping[K, V] | Iterable[tuple[K, V]] | None = None, **kwargs: V) -> None:
        """Update from dict, iterable of pairs, or keyword arguments."""
        if other is not None:
            if isinstance(other, Mapping):
                for k, v in other.items():
                    self[k] = v
            else:
                for k, v in other:
                    self[k] = v
        for k, v in kwargs.items():
            self[k] = v  # type: ignore[index]

    def setdefault(self, key: K, default: V | None = None) -> V | None:
        """Return value for key, setting default if not present."""
        if key in self._dict:
            return self._dict[key].value
        self[key] = default  # type: ignore[assignment]
        return default


class _DequeDictKeysView(KeysView[K]):
    __slots__ = ("_dd",)

    def __init__(self, dd: DequeDict[K, V]) -> None:
        self._dd = dd

    def __len__(self) -> int:
        return len(self._dd)

    def __iter__(self) -> Iterator[K]:
        return iter(self._dd)

    def __contains__(self, key: object) -> bool:
        return key in self._dd


class _DequeDictValuesView(ValuesView[V]):
    __slots__ = ("_dd",)

    def __init__(self, dd: DequeDict[K, V]) -> None:
        self._dd = dd

    def __len__(self) -> int:
        return len(self._dd)

    def __iter__(self) -> Iterator[V]:
        for node in self._dd._iter_nodes():
            yield node.value

    def __contains__(self, value: object) -> bool:
        return any(node.value == value for node in self._dd._iter_nodes())


class _DequeDictItemsView(ItemsView[K, V]):
    __slots__ = ("_dd",)

    def __init__(self, dd: DequeDict[K, V]) -> None:
        self._dd = dd

    def __len__(self) -> int:
        return len(self._dd)

    def __iter__(self) -> Iterator[tuple[K, V]]:
        for node in self._dd._iter_nodes():
            yield (node.key, node.value)

    def __contains__(self, item: object) -> bool:
        if not isinstance(item, tuple) or len(item) != 2:
            return False
        k, v = item
        if k not in self._dd:
            return False
        return self._dd[k] == v


class DefaultDequeDict(DequeDict[K, V]):
    """DequeDict with default_factory for missing keys, like collections.defaultdict."""

    __slots__ = ("default_factory",)

    def __init__(
        self,
        default_factory: Callable[[], V] | None = None,
        items: Mapping[K, V] | Iterable[tuple[K, V]] | None = None,
    ) -> None:
        self.default_factory = default_factory
        super().__init__(items)

    def __missing__(self, key: K) -> V:
        if self.default_factory is None:
            raise KeyError(key)
        value = self.default_factory()
        self[key] = value
        return value

    def __getitem__(self, key: K) -> V:
        try:
            return super().__getitem__(key)
        except KeyError:
            return self.__missing__(key)

    def __repr__(self) -> str:
        return f"DefaultDequeDict({self.default_factory}, {list(self.items())!r})"

    def copy(self) -> DefaultDequeDict[K, V]:
        return DefaultDequeDict(self.default_factory, self.items())


# Use C extension if available (disable with NOC=1 environment variable)
if not TYPE_CHECKING and not os.getenv("NOC"):
    with suppress(ImportError):
        from dequedict._dequedict import DequeDict



"""Typing stubs for dequedict."""
from typing import TypeVar, Iterator, ItemsView, KeysView, ValuesView, Callable, overload
from collections.abc import Iterable, Mapping

K = TypeVar("K")
V = TypeVar("V")

class DequeDict(Mapping[K, V]):
    """Ordered dictionary with O(1) deque operations at both ends.

    Combines dict interface with deque-like operations:
    - O(1) popleft/popleftitem/peekleft/peekleftitem
    - O(1) pop/popitem/peek/peekitem (right side)
    - O(1) appendleft (insert at front)
    - O(1) lookup by key
    - Maintains insertion order
    """

    def __init__(self, items: Mapping[K, V] | Iterable[tuple[K, V]] | None = None) -> None: ...
    def __len__(self) -> int: ...
    def __contains__(self, key: object) -> bool: ...
    def __getitem__(self, key: K) -> V: ...
    def __setitem__(self, key: K, value: V) -> None: ...
    def __delitem__(self, key: K) -> None: ...
    def __iter__(self) -> Iterator[K]: ...
    def __reversed__(self) -> Iterator[K]: ...
    def __repr__(self) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

    # Deque-like operations - O(1)
    def peekleft(self) -> V:
        """Return first value without removing - O(1)."""
        ...

    def peekleftitem(self) -> tuple[K, V]:
        """Return first (key, value) without removing - O(1)."""
        ...

    def peekleftkey(self) -> K:
        """Return first key without removing - O(1)."""
        ...

    def peek(self) -> V:
        """Return last value without removing - O(1)."""
        ...

    def peekitem(self) -> tuple[K, V]:
        """Return last (key, value) without removing - O(1)."""
        ...

    def popleft(self) -> V:
        """Remove and return first value - O(1)."""
        ...

    def popleftitem(self) -> tuple[K, V]:
        """Remove and return first (key, value) - O(1)."""
        ...

    @overload
    def pop(self) -> V: ...
    @overload
    def pop(self, key: K) -> V: ...
    @overload
    def pop(self, key: K, default: V) -> V: ...

    def popitem(self) -> tuple[K, V]:
        """Remove and return last (key, value) - O(1)."""
        ...

    def appendleft(self, key: K, value: V) -> None:
        """Insert (key, value) at front - O(1)."""
        ...

    def move_to_end(self, key: K, last: bool = True) -> None:
        """Move key to front (last=False) or back (last=True) - O(1)."""
        ...

    # Dict-like operations
    @overload
    def get(self, key: K) -> V | None: ...
    @overload
    def get(self, key: K, default: V) -> V: ...

    def keys(self) -> KeysView[K]:
        """D.keys() -> view of keys in order."""
        ...

    def values(self) -> ValuesView[V]:
        """D.values() -> view of values in order."""
        ...

    def items(self) -> ItemsView[K, V]:
        """D.items() -> view of (key, value) in order."""
        ...

    def clear(self) -> None:
        """D.clear() -- remove all items."""
        ...

    def copy(self) -> DequeDict[K, V]:
        """D.copy() -> a shallow copy."""
        ...

    def update(self, other: Mapping[K, V] | Iterable[tuple[K, V]] | None = None, **kwargs: V) -> None:
        """D.update([E, ]**F)."""
        ...

    @overload
    def setdefault(self, key: K) -> V | None: ...
    @overload
    def setdefault(self, key: K, default: V) -> V: ...


class DefaultDequeDict(DequeDict[K, V]):
    """DequeDict with default_factory for missing keys."""

    default_factory: Callable[[], V] | None

    def __init__(
        self,
        default_factory: Callable[[], V] | None = None,
        items: Mapping[K, V] | Iterable[tuple[K, V]] | None = None,
    ) -> None: ...

    def __missing__(self, key: K) -> V: ...
    def copy(self) -> DefaultDequeDict[K, V]: ...




"""Tests for DequeDict following SETUP → EXPECTED → ACT → ASSERT pattern."""
from __future__ import annotations
import pytest
from dequedict import DequeDict, DefaultDequeDict


class TestDequeDictInit:
    """Tests for DequeDict initialization."""

    def test_init_empty_creates_empty_dequedict(self):
        # ACT
        dd = DequeDict()

        # ASSERT
        expected_len = 0
        assert len(dd) == expected_len

    def test_init_from_dict_preserves_items(self):
        # SETUP
        source = {"a": 1, "b": 2, "c": 3}

        # EXPECTED
        expected_len = 3
        expected_a = 1
        expected_b = 2

        # ACT
        dd = DequeDict(source)

        # ASSERT
        assert len(dd) == expected_len
        assert dd["a"] == expected_a
        assert dd["b"] == expected_b

    def test_init_from_pairs_preserves_order(self):
        # SETUP
        pairs = [("x", 10), ("y", 20), ("z", 30)]

        # EXPECTED
        expected_keys = ["x", "y", "z"]
        expected_first_key = "x"
        expected_last_key = "z"

        # ACT
        dd = DequeDict(pairs)

        # ASSERT
        assert list(dd.keys()) == expected_keys
        assert dd.peekleftkey() == expected_first_key
        assert dd.peekitem()[0] == expected_last_key


class TestDequeDictGetSet:
    """Tests for getting and setting items."""

    def test_setitem_adds_new_key_at_end(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_last_key = "b"
        expected_value = 2

        # ACT
        dd["b"] = 2

        # ASSERT
        assert dd.peekitem()[0] == expected_last_key
        assert dd["b"] == expected_value

    def test_setitem_updates_existing_key_value(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_value = 99
        expected_order = ["a", "b"]

        # ACT
        dd["a"] = 99

        # ASSERT
        assert dd["a"] == expected_value
        assert list(dd.keys()) == expected_order

    def test_getitem_raises_keyerror_for_missing_key(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # ACT & ASSERT
        with pytest.raises(KeyError):
            _ = dd["missing"]

    def test_delitem_removes_key(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_len = 2
        expected_keys = ["a", "c"]

        # ACT
        del dd["b"]

        # ASSERT
        assert len(dd) == expected_len
        assert list(dd.keys()) == expected_keys
        assert "b" not in dd


class TestDequeDictContains:
    """Tests for membership checking."""

    def test_contains_returns_true_for_existing_key(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_result = True

        # ACT
        result = "a" in dd

        # ASSERT
        assert result == expected_result

    def test_contains_returns_false_for_missing_key(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_result = False

        # ACT
        result = "missing" in dd

        # ASSERT
        assert result == expected_result


class TestDequeDictPeek:
    """Tests for peek operations."""

    def test_peekleft_returns_first_value(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_value = 1

        # ACT
        result = dd.peekleft()

        # ASSERT
        assert result == expected_value

    def test_peekleftitem_returns_first_pair(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_pair = ("a", 1)

        # ACT
        result = dd.peekleftitem()

        # ASSERT
        assert result == expected_pair

    def test_peekleftkey_returns_first_key(self):
        # SETUP
        dd = DequeDict([("first", 100), ("second", 200)])

        # EXPECTED
        expected_key = "first"

        # ACT
        result = dd.peekleftkey()

        # ASSERT
        assert result == expected_key

    def test_peek_returns_last_value(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_value = 3

        # ACT
        result = dd.peek()

        # ASSERT
        assert result == expected_value

    def test_peekitem_returns_last_pair(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_pair = ("b", 2)

        # ACT
        result = dd.peekitem()

        # ASSERT
        assert result == expected_pair

    def test_peekleft_raises_on_empty(self):
        # SETUP
        dd = DequeDict()

        # ACT & ASSERT
        with pytest.raises(IndexError, match="empty"):
            dd.peekleft()

    def test_peek_raises_on_empty(self):
        # SETUP
        dd = DequeDict()

        # ACT & ASSERT
        with pytest.raises(IndexError, match="empty"):
            dd.peek()


class TestDequeDictPop:
    """Tests for pop operations."""

    def test_popleft_removes_and_returns_first_value(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_value = 1
        expected_remaining_keys = ["b", "c"]

        # ACT
        result = dd.popleft()

        # ASSERT
        assert result == expected_value
        assert list(dd.keys()) == expected_remaining_keys

    def test_popleftitem_removes_and_returns_first_pair(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_pair = ("a", 1)
        expected_len = 1

        # ACT
        result = dd.popleftitem()

        # ASSERT
        assert result == expected_pair
        assert len(dd) == expected_len

    def test_pop_without_key_removes_last(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_value = 3
        expected_remaining_keys = ["a", "b"]

        # ACT
        result = dd.pop()

        # ASSERT
        assert result == expected_value
        assert list(dd.keys()) == expected_remaining_keys

    def test_pop_with_key_removes_specific_item(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_value = 2
        expected_remaining_keys = ["a", "c"]

        # ACT
        result = dd.pop("b")

        # ASSERT
        assert result == expected_value
        assert list(dd.keys()) == expected_remaining_keys

    def test_pop_with_default_returns_default_for_missing(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_value = "default"

        # ACT
        result = dd.pop("missing", "default")

        # ASSERT
        assert result == expected_value

    def test_popitem_removes_and_returns_last_pair(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_pair = ("b", 2)
        expected_len = 1

        # ACT
        result = dd.popitem()

        # ASSERT
        assert result == expected_pair
        assert len(dd) == expected_len

    def test_popleft_raises_on_empty(self):
        # SETUP
        dd = DequeDict()

        # ACT & ASSERT
        with pytest.raises(IndexError):
            dd.popleft()

    def test_popitem_raises_on_empty(self):
        # SETUP
        dd = DequeDict()

        # ACT & ASSERT
        with pytest.raises(KeyError):
            dd.popitem()


class TestDequeDictAppendLeft:
    """Tests for appendleft operation."""

    def test_appendleft_inserts_at_front(self):
        # SETUP
        dd = DequeDict([("b", 2), ("c", 3)])

        # EXPECTED
        expected_keys = ["a", "b", "c"]
        expected_first_key = "a"

        # ACT
        dd.appendleft("a", 1)

        # ASSERT
        assert list(dd.keys()) == expected_keys
        assert dd.peekleftkey() == expected_first_key

    def test_appendleft_raises_for_existing_key(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # ACT & ASSERT
        with pytest.raises(KeyError, match="already exists"):
            dd.appendleft("a", 99)


class TestDequeDictMoveToEnd:
    """Tests for move_to_end operation."""

    def test_move_to_end_moves_to_back(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_keys = ["b", "c", "a"]

        # ACT
        dd.move_to_end("a")

        # ASSERT
        assert list(dd.keys()) == expected_keys

    def test_move_to_end_with_last_false_moves_to_front(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_keys = ["c", "a", "b"]

        # ACT
        dd.move_to_end("c", last=False)

        # ASSERT
        assert list(dd.keys()) == expected_keys

    def test_move_to_end_raises_for_missing_key(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # ACT & ASSERT
        with pytest.raises(KeyError):
            dd.move_to_end("missing")


class TestDequeDictGet:
    """Tests for get method."""

    def test_get_returns_value_for_existing_key(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_value = 1

        # ACT
        result = dd.get("a")

        # ASSERT
        assert result == expected_value

    def test_get_returns_none_for_missing_key(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_value = None

        # ACT
        result = dd.get("missing")

        # ASSERT
        assert result == expected_value

    def test_get_returns_default_for_missing_key(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_value = "default"

        # ACT
        result = dd.get("missing", "default")

        # ASSERT
        assert result == expected_value


class TestDequeDictViews:
    """Tests for keys, values, and items views."""

    def test_keys_returns_keys_in_order(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_keys = ["a", "b", "c"]

        # ACT
        result = list(dd.keys())

        # ASSERT
        assert result == expected_keys

    def test_values_returns_values_in_order(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_values = [1, 2, 3]

        # ACT
        result = list(dd.values())

        # ASSERT
        assert result == expected_values

    def test_items_returns_pairs_in_order(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_items = [("a", 1), ("b", 2)]

        # ACT
        result = list(dd.items())

        # ASSERT
        assert result == expected_items

    def test_keys_view_contains(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # ACT & ASSERT
        assert "a" in dd.keys()
        assert "c" not in dd.keys()

    def test_items_view_contains(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # ACT & ASSERT
        assert ("a", 1) in dd.items()
        assert ("a", 99) not in dd.items()


class TestDequeDictIteration:
    """Tests for iteration."""

    def test_iter_yields_keys_in_order(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_keys = ["a", "b", "c"]

        # ACT
        result = list(dd)

        # ASSERT
        assert result == expected_keys

    def test_reversed_yields_keys_in_reverse_order(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2), ("c", 3)])

        # EXPECTED
        expected_keys = ["c", "b", "a"]

        # ACT
        result = list(reversed(dd))

        # ASSERT
        assert result == expected_keys


class TestDequeDictClearCopy:
    """Tests for clear and copy operations."""

    def test_clear_removes_all_items(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_len = 0

        # ACT
        dd.clear()

        # ASSERT
        assert len(dd) == expected_len

    def test_copy_creates_shallow_copy(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])

        # EXPECTED
        expected_items = [("a", 1), ("b", 2)]

        # ACT
        copy = dd.copy()
        dd["c"] = 3

        # ASSERT
        assert list(copy.items()) == expected_items
        assert "c" not in copy


class TestDequeDictUpdate:
    """Tests for update method."""

    def test_update_from_dict(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_len = 2
        expected_b = 2

        # ACT
        dd.update({"b": 2})

        # ASSERT
        assert len(dd) == expected_len
        assert dd["b"] == expected_b

    def test_update_from_pairs(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_keys = ["a", "b", "c"]

        # ACT
        dd.update([("b", 2), ("c", 3)])

        # ASSERT
        assert list(dd.keys()) == expected_keys

    def test_update_with_kwargs(self):
        # SETUP
        dd = DequeDict()

        # EXPECTED
        expected_x = 10

        # ACT
        dd.update(x=10, y=20)

        # ASSERT
        assert dd["x"] == expected_x


class TestDequeDictSetDefault:
    """Tests for setdefault method."""

    def test_setdefault_returns_existing_value(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # EXPECTED
        expected_value = 1

        # ACT
        result = dd.setdefault("a", 99)

        # ASSERT
        assert result == expected_value
        assert dd["a"] == expected_value

    def test_setdefault_sets_and_returns_default(self):
        # SETUP
        dd = DequeDict()

        # EXPECTED
        expected_value = "default"

        # ACT
        result = dd.setdefault("a", "default")

        # ASSERT
        assert result == expected_value
        assert dd["a"] == expected_value


class TestDequeDictEquality:
    """Tests for equality comparison."""

    def test_equal_to_dict_with_same_contents(self):
        # SETUP
        dd = DequeDict([("a", 1), ("b", 2)])
        d = {"a": 1, "b": 2}

        # ACT & ASSERT
        assert dd == d

    def test_not_equal_to_dict_with_different_contents(self):
        # SETUP
        dd = DequeDict([("a", 1)])
        d = {"a": 2}

        # ACT & ASSERT
        assert dd != d

    def test_equal_to_another_dequedict(self):
        # SETUP
        dd1 = DequeDict([("a", 1), ("b", 2)])
        dd2 = DequeDict([("a", 1), ("b", 2)])

        # ACT & ASSERT
        assert dd1 == dd2


class TestDequeDictRepr:
    """Tests for repr."""

    def test_repr_empty(self):
        # SETUP
        dd = DequeDict()

        # EXPECTED
        expected_repr = "DequeDict()"

        # ACT
        result = repr(dd)

        # ASSERT
        assert result == expected_repr

    def test_repr_with_items(self):
        # SETUP
        dd = DequeDict([("a", 1)])

        # ACT
        result = repr(dd)

        # ASSERT
        assert "DequeDict" in result
        assert "a" in result


class TestDefaultDequeDict:
    """Tests for DefaultDequeDict with default_factory."""

    def test_default_factory_list_creates_on_access(self):
        # SETUP
        dd = DefaultDequeDict(list)

        # EXPECTED
        expected_value = [1, 2]

        # ACT
        dd["a"].append(1)
        dd["a"].append(2)

        # ASSERT
        assert dd["a"] == expected_value

    def test_default_factory_int_starts_at_zero(self):
        # SETUP
        dd = DefaultDequeDict(int)

        # EXPECTED
        expected_value = 5

        # ACT
        dd["x"] += 5

        # ASSERT
        assert dd["x"] == expected_value

    def test_no_factory_raises_keyerror(self):
        # SETUP
        dd = DefaultDequeDict()

        # ACT & ASSERT
        with pytest.raises(KeyError):
            _ = dd["missing"]

    def test_deque_ops_still_work(self):
        # SETUP
        dd = DefaultDequeDict(int)
        dd["a"] = 1
        dd["b"] = 2

        # EXPECTED
        expected_first = 1

        # ACT
        result = dd.popleft()

        # ASSERT
        assert result == expected_first
        assert "a" not in dd

    def test_copy_preserves_factory(self):
        # SETUP
        dd = DefaultDequeDict(list)
        dd["a"].append(1)

        # ACT
        copy = dd.copy()
        copy["b"].append(2)

        # ASSERT
        assert copy["b"] == [2]
        assert copy.default_factory is list


if __name__ == "__main__":
    pytest.main([__file__, "-vv"])




from setuptools import Extension, setup

extensions = [
    Extension(
        "dequedict._dequedict",
        sources=["dequedict/_dequedict.c"],
        extra_compile_args=["-O3", "-Wall"],
    ),
]

setup(ext_modules=extensions)

